from setuptools import setup
setup(
  name = 'pytradfri',
  packages = ['pytradfri'],
  version = '0.5',
  description = 'A python library for communicating with the IKEA Tradfri Gateway',
  author = 'balloob, ggravlingen',
  author_email = 'no@email.com',
  url = 'https://github.com/ggravlingen/pytradfri',
  download_url = 'https://github.com/ggravlingen/pytradfri/archive/v0.5.zip'
)
